@@ -4,12 +4,11 @@
 Database of DNA sequences.
 """
 
-__version__ = '0.1.0'
+__version__ = '0.3.0'
 
 from .config import *
 from .errors import *
 from .model import *
-from .protocols import *
 from .fields import *
-from .combos import *
+from .group_by import *
 from .utils import *
