@@ -1,12 +1,10 @@
 #!/usr/bin/env python3
 
 import re
-from Bio.Seq import Seq
-from Bio.Alphabet import generic_dna
+from stepwise import Quantity
 from more_itertools import split_when
 from .errors import ParseError, only_raise
 
-DnaSeq = lambda x: Seq(normalize_seq(x), generic_dna)
 no_default = object()
 
 def normalize_seq(raw_seq):
@@ -21,7 +19,7 @@ def normalize_seq(raw_seq):
     return seq.upper()
 
 def get_tag_prefixes(*args):
-    from .model import Construct
+    from .model import Reagent
 
     def add(prefixes, cls):
         if cls.tag_prefix:
@@ -30,7 +28,7 @@ def get_tag_prefixes(*args):
             add(prefixes, subcls)
 
     prefixes = set()
-    for cls in args or [Construct]:
+    for cls in args or [Reagent]:
         add(prefixes, cls)
     
     return prefixes
@@ -54,63 +52,6 @@ def parse_tag(tag_str):
     else:
         raise ParseError(f"expected a tag (e.g. 'p100'), not {tag_str!r}")
 
-@only_raise(ParseError)
-def parse_params(params_str, *, sep=r'\s', on_duplicate=None, on_unmatched=None):
-    params = {}
-    param_pattern = fr'''\s*
-            (?P<key>\w+)=(
-              (?P<value>[^"'][^{sep}]*)|
-              (?P<quote>["'])(?P<value_quoted>.*?)(?P=quote)
-            )
-            ({sep}|$)
-    '''
-
-    unmatched = set(range(len(params_str)))
-
-    for m in re.finditer(param_pattern, params_str, re.VERBOSE):
-        key = m.group('key')
-        value = m.group('value') or m.group('value_quoted')
-
-        if key in params:
-            if on_duplicate:
-                on_duplicate(params, key, value)
-            else:
-                raise ParseError(f"duplicate key {key!r} in {params_str!r}")
-
-        params[key] = value
-        unmatched -= set(range(m.start(), m.end()))
-
-    if unmatched:
-        if on_unmatched:
-            on_unmatched(params_str, unmatched)
-        else:
-            groups = split_when(
-                    sorted(unmatched), 
-                    lambda a, b: b != a + 1
-            )
-            substrs = ', '.join(
-                    repr(params_str[x[0]:x[-1]+1])
-                    for x in groups
-            )
-            if substrs == repr(params_str):
-                raise ParseError(f"can't parse {params_str!r}")
-            else:
-                raise ParseError(f"can't parse {params_str!r}: didn't expect {substrs}")
-
-    return params
-
-def parse_param(params, key, pattern, default=no_default):
-    if key not in params:
-        if default is not no_default:
-            return default
-        else:
-            raise ParseError(f"missing required {key!r} parameter")
-
-    if m := re.fullmatch(pattern, p := params[key]):
-        return m.groups() or m.group()
-    else:
-        raise ParseError(f"expected {key}≈{pattern!r}, found {p!r}")
-
 @only_raise(ParseError)
 def parse_bool(bool_str):
     if bool_str.lower() in ('1', 'y', 'yes', 'true'):
@@ -206,6 +147,42 @@ def parse_conc_ng_uL(conc_str, mw):
     else:
         raise ParseError(f"can't interpret {conc_str!r} as a concentration, did you forget a unit?")
 
+@only_raise(ParseError)
+def convert_conc_unit(conc, mw, new_unit):
+    molar_conversion_factors = {
+            'pM': 1e12,
+            'nM': 1e9,
+            'uM': 1e6,
+            'µM': 1e6,
+            'mM': 1e3,
+            'M': 1,
+    }
+    mass_volume_conversion_factors = {
+            'ng/uL': 1e3,
+            'ng/µL': 1e3,
+            'mg/mL': 1,
+    }
+
+    def pick_conversion_factors():
+        if mw is not None:
+            return {
+                    **molar_conversion_factors,
+                    **{
+                        k: mw * v
+                        for k, v in mass_volume_conversion_factors.items()
+                    },
+            }
+
+        if conc.unit in molar_conversion_factors:
+            return molar_conversion_factors
+
+        if conc.unit in mass_volume_conversion_factors:
+            return mass_volume_conversion_factors
+
+        return {}
+
+    return conc.convert_unit(new_unit, pick_conversion_factors())
+
 @only_raise(ParseError)
 def parse_size_bp(size_str):
     bp_parsers = [
@@ -224,8 +201,4 @@ def parse_size_bp(size_str):
 
     raise ParseError(f"can't interpret {size_str!r} as a size in base pairs, did you forget a unit?")
 
-def unanimous(values):
-    from itertools import groupby
-    from more_itertools import one
-    return one((x[0] for x in groupby(values)))
 
